==> krb4-0.7.tar.gz: krb4-0.7/lib/krb/in_tkt.c <==
/*
 * This software may now be redistributed outside the US.
 *
 */

/* 
  Copyright (C) 1989 by the Massachusetts Institute of Technology

   Export of this software from the United States of America is assumed
   to require a specific license from the United States Government.
   It is the responsibility of any person or organization contemplating
   export to obtain such a license before exporting.

WITHIN THAT CONSTRAINT, permission to use, copy, modify, and
distribute this software and its documentation for any purpose and
without fee is hereby granted, provided that the above copyright
notice appear in all copies and that both that copyright notice and
this permission notice appear in supporting documentation, and that
the name of M.I.T. not be used in advertising or publicity pertaining
to distribution of the software without specific, written prior
permission.  M.I.T. makes no representations about the suitability of
this software for any purpose.  It is provided "as is" without express
or implied warranty.

  */

#include "krb_locl.h"

RCSID("$Header: /src/packages/kth-krb/SourceRepository/krb4/lib/krb/in_tkt.c,v 1.6 1996/03/18 16:32:43 bg Exp $");

#include <sys/file.h>
#include <sys/types.h>
#include <sys/stat.h>
#ifdef TKT_SHMEM
#include <sys/param.h>
#endif

/*
 * in_tkt() is used to initialize the ticket store.  It creates the
 * file to contain the tickets and writes the given user's name "pname"
 * and instance "pinst" in the file.  in_tkt() returns KSUCCESS on
 * success, or KFAILURE if something goes wrong.
 */

int
in_tkt(char *pname, char *pinst)
{
    int tktfile;
    uid_t me, metoo;
    struct stat buf;
    int count;
    char *file = TKT_FILE;
    int fd;
    register int i;
    char charbuf[BUFSIZ];
#ifdef TKT_SHMEM
    char shmidname[MaxPathLen];
#endif /* TKT_SHMEM */

    me = getuid ();
    metoo = geteuid();
    if (lstat(file,&buf) == 0) {
	if (buf.st_uid != me || !(buf.st_mode & S_IFREG) ||
	    buf.st_mode & 077) {
#ifndef KRB_NDEB
	    if (krb_debug)
		fprintf(stderr,"Error initializing %s",file);
#endif
	    return(KFAILURE);
	}
	/* file already exists, and permissions appear ok, so nuke it */
	if ((fd = open(file, O_RDWR, 0)) < 0)
	    goto out; /* can't zero it, but we can still try truncating it */

	bzero(charbuf, sizeof(charbuf));

	for (i = 0; i < buf.st_size; i += sizeof(charbuf))
	    if (write(fd, charbuf, sizeof(charbuf)) != sizeof(charbuf)) {
		(void) fsync(fd);
		(void) close(fd);
		goto out;
	    }
	
	(void) fsync(fd);
	(void) close(fd);
    }
 out:
    /* arrange so the file is owned by the ruid
       (swap real & effective uid if necessary).
       This isn't a security problem, since the ticket file, if it already
       exists, has the right uid (== ruid) and mode. */
    if (me != metoo) {
	if (k_setreuid(metoo, me) < 0) {
	    /* can't switch??? barf! */
#ifndef KRB_NDEB
	    if (krb_debug)
		perror("in_tkt: setreuid");
#endif
	    return(KFAILURE);
	} else
	  {
#ifndef KRB_NDEB
	    if (krb_debug)
	      fprintf(stderr, "swapped UID's %d and %d\n",(int)metoo,(int)me);
#endif
	  }
    }
    if ((tktfile = creat(file,0600)) < 0) {
#ifndef KRB_NDEB
	if (krb_debug)
	    fprintf(stderr,"Error initializing %s",TKT_FILE);
#endif
        return(KFAILURE);
    }
    if (me != metoo) {
	if (k_setreuid(me, metoo) < 0) {
	    /* can't switch??? barf! */
#ifndef KRB_NDEB
	    if (krb_debug)
		perror("in_tkt: setreuid2");
#endif
	    return(KFAILURE);
	} else
	  {
#ifndef KRB_NDEB
	    if (krb_debug)
	      fprintf(stderr, "swapped UID's %d and %d\n",(int)me,(int)metoo);
#endif
	  }
    }
    if (lstat(file,&buf) < 0) {
#ifndef KRB_NDEB
	if (krb_debug)
	    fprintf(stderr,"Error initializing %s",TKT_FILE);
#endif
        return(KFAILURE);
    }

    if (buf.st_uid != me || !(buf.st_mode & S_IFREG) ||
        buf.st_mode & 077) {
#ifndef KRB_NDEB
	if (krb_debug)
	    fprintf(stderr,"Error initializing %s",TKT_FILE);
#endif
        return(KFAILURE);
    }

    count = strlen(pname)+1;
    if (write(tktfile,pname,count) != count) {
        (void) close(tktfile);
        return(KFAILURE);
    }
    count = strlen(pinst)+1;
    if (write(tktfile,pinst,count) != count) {
        (void) close(tktfile);
        return(KFAILURE);
    }
    (void) close(tktfile);
#ifdef TKT_SHMEM
    (void) strcpy(shmidname, file);
    (void) strcat(shmidname, ".shm");
    return(krb_shm_create(shmidname));
#else /* !TKT_SHMEM */
    return(KSUCCESS);
#endif /* TKT_SHMEM */
}

==> krb4-0.8.tar.gz: krb4-0.8/lib/krb/in_tkt.c <==
/*
 * This software may now be redistributed outside the US.
 *
 */

/* 
  Copyright (C) 1989 by the Massachusetts Institute of Technology

   Export of this software from the United States of America is assumed
   to require a specific license from the United States Government.
   It is the responsibility of any person or organization contemplating
   export to obtain such a license before exporting.

WITHIN THAT CONSTRAINT, permission to use, copy, modify, and
distribute this software and its documentation for any purpose and
without fee is hereby granted, provided that the above copyright
notice appear in all copies and that both that copyright notice and
this permission notice appear in supporting documentation, and that
the name of M.I.T. not be used in advertising or publicity pertaining
to distribution of the software without specific, written prior
permission.  M.I.T. makes no representations about the suitability of
this software for any purpose.  It is provided "as is" without express
or implied warranty.

  */

#include "krb_locl.h"

RCSID("$Id: in_tkt.c,v 1.9 1996/04/28 17:34:19 d91-jda Exp $");

#include <sys/file.h>
#include <sys/types.h>
#include <sys/stat.h>
#ifdef TKT_SHMEM
#include <sys/param.h>
#endif

/*
 * in_tkt() is used to initialize the ticket store.  It creates the
 * file to contain the tickets and writes the given user's name "pname"
 * and instance "pinst" in the file.  in_tkt() returns KSUCCESS on
 * success, or KFAILURE if something goes wrong.
 */

#ifdef KRB_NDEB
#define DEBUG(X)
#else
#define DEBUG(X) if(krb_debug){ X; }
#endif

int
in_tkt(char *pname, char *pinst)
{
    int tktfile;
    uid_t uid;
    struct stat buf;
    int count;
    char *file = TKT_FILE;
    int fd;
    register int i;
    char charbuf[BUFSIZ];
#ifdef TKT_SHMEM
    char shmidname[MaxPathLen];
#endif /* TKT_SHMEM */

    uid = getuid ();
    if (lstat(file,&buf) == 0) {
	if (buf.st_uid != uid || !(buf.st_mode & S_IFREG) ||
	    buf.st_mode & 077) {
	    DEBUG(fprintf(stderr, "Error initializing %s",file));
	    return(KFAILURE);
	}
	/* file already exists, and permissions appear ok, so nuke it */
	if ((fd = open(file, O_RDWR, 0)) < 0)
	    goto out; /* can't zero it, but we can still try truncating it */

	bzero(charbuf, sizeof(charbuf));

	for (i = 0; i < buf.st_size; i += sizeof(charbuf))
	    if (write(fd, charbuf, sizeof(charbuf)) != sizeof(charbuf)) {
		(void) fsync(fd);
		(void) close(fd);
		goto out;
	    }
	
	(void) fsync(fd);
	(void) close(fd);
    }
 out:
    if ((tktfile = open(file, O_WRONLY|O_CREAT|O_TRUNC, 0600)) < 0) {
	DEBUG(fprintf(stderr,"Error initializing %s",TKT_FILE));
        return(KFAILURE);
    }
    if (lstat(file,&buf) < 0) {
	DEBUG(fprintf(stderr,"Error initializing %s",TKT_FILE));
        return(KFAILURE);
    }
    
    if (buf.st_uid != uid || !(buf.st_mode & S_IFREG) ||
        buf.st_mode & 077) {
	DEBUG(fprintf(stderr,"Error initializing %s",TKT_FILE));
        return(KFAILURE);
    }

    count = strlen(pname)+1;
    if (write(tktfile,pname,count) != count) {
        (void) close(tktfile);
        return(KFAILURE);
    }
    count = strlen(pinst)+1;
    if (write(tktfile,pinst,count) != count) {
        (void) close(tktfile);
        return(KFAILURE);
    }
    (void) close(tktfile);
#ifdef TKT_SHMEM
    (void) strcpy(shmidname, file);
    (void) strcat(shmidname, ".shm");
    return(krb_shm_create(shmidname));
#else /* !TKT_SHMEM */
    return(KSUCCESS);
#endif /* TKT_SHMEM */
}
